@@ -174,6 +174,8 @@ def get_channel_calibrations(device, device_info, incrementer):
 
     def element_fn(i):
         nonlocal supports_calibration
+        if not supports_calibration:
+            return None
         try:
             return device.get_channel_calibration(i)
         except asphodel.AsphodelError as e:
@@ -508,16 +510,12 @@ def get_device_info_actual(device, device_logger, allow_reconnect, diskcache,
                 diskcache.set(setting_key, setting_info)
                 if nvm_hash is not None:
                     diskcache.set(nvm_key, nvm_info)
-
-    except Exception:
-        device_logger.exception("Unhandled exception fetching device info")
-        return {}
     finally:
         if progress_pipe:
             progress_pipe.close()
 
-    device_logger.info("Fetched device information using %s commands",
-                       finished_commands)
+    device_logger.debug("Fetched device information using %s commands",
+                        finished_commands)
 
     return merged_dict
 
@@ -569,6 +567,10 @@ def get_active_scan_info(device, remote, diskcache=None):
 def get_device_info(device, allow_reconnect, diskcache=None,
                     progress_pipe=None):
     device_logger = hyperborea.proxy.get_device_logger(logger, device)
-    return get_device_info_actual(
-        device, device_logger, allow_reconnect, diskcache, progress_pipe,
-        default_setting_getters, [get_nvm])
+    try:
+        return get_device_info_actual(
+            device, device_logger, allow_reconnect, diskcache, progress_pipe,
+            default_setting_getters, [get_nvm])
+    except Exception:
+        device_logger.exception("Unhandled exception fetching device info")
+        return {}
