@@ -19,7 +19,7 @@ class StringLengthValidator(QtGui.QValidator):
     def validate(self, input_text, pos):
         try:
             utf_bytes = input_text.encode("UTF-8")
-        except:
+        except Exception:
             return (QtGui.QValidator.State.Invalid, input_text, pos)
 
         if len(utf_bytes) <= self.max_length:
@@ -292,7 +292,7 @@ class SettingWidget(QtWidgets.QWidget):
 
         self.lineedit = QtWidgets.QLineEdit(self)
         self.regexp = QtCore.QRegExp("[1-9, ]?")
-        self.validator = QtGui.QRegExpValidator(self)
+        self.validator = QtGui.QRegExpValidator(self.regexp, self)
         self.lineedit.setValidator(self.validator)
 
         len_offset = s.length_nvm_word * 4 + s.length_nvm_word_byte
@@ -539,7 +539,7 @@ class SettingWidget(QtWidgets.QWidget):
 
         self.lineedit = QtWidgets.QLineEdit(self)
         self.regexp = QtCore.QRegExp("[1-9, .e\-]?")
-        self.validator = QtGui.QRegExpValidator(self)
+        self.validator = QtGui.QRegExpValidator(self.regexp, self)
         self.lineedit.setValidator(self.validator)
 
         scaled_min = s.minimum * s.scale + s.offset
