@@ -25,7 +25,7 @@ instances = {}
 
 def start_streaming(device, *args, **kwargs):
     device_logger = hyperborea.proxy.get_device_logger(logger, device)
-    device_logger.info("Starting streaming instance")
+    device_logger.debug("Starting streaming instance")
 
     if device in instances:
         stop_streaming(device)
@@ -34,8 +34,9 @@ def start_streaming(device, *args, **kwargs):
     instances[device] = instance
     instance.start()
 
-    hyperborea.proxy.register_local_cleanup(stop_streaming, device)
-    hyperborea.proxy.register_local_cleanup(join_streaming, device, instance)
+    hyperborea.proxy.register_device_cleanup(device, stop_streaming)
+    hyperborea.proxy.register_device_cleanup(None, join_streaming, instance,
+                                             device_logger)
 
 
 def stop_streaming(device):
@@ -45,20 +46,21 @@ def stop_streaming(device):
     instances[device].stop()
 
     device_logger = hyperborea.proxy.get_device_logger(logger, device)
-    device_logger.info("Stopped streaming instance")
+    device_logger.debug("Stopped streaming instance")
 
     del instances[device]
 
-    hyperborea.proxy.unregister_local_cleanup(stop_streaming, device)
+    hyperborea.proxy.unregister_device_cleanup(device, stop_streaming)
 
 
-def join_streaming(device, instance):  # called during cleanup, on proxy exit
+def join_streaming(_device, instance, device_logger):
+    # called during cleanup, on proxy exit
+
     instance.join()
+    device_logger.debug("Finished streaming instance")
 
-    device_logger = hyperborea.proxy.get_device_logger(logger, device)
-    device_logger.info("Finished streaming instance")
-
-    hyperborea.proxy.unregister_local_cleanup(join_streaming, device, instance)
+    hyperborea.proxy.unregister_device_cleanup(None, join_streaming,
+                                               instance, device_logger)
 
 
 def get_valid_filename(s):
@@ -280,7 +282,7 @@ class StreamWriter:
             # create a .upload file to mark it for uploading
             path, name = os.path.split(filename)
             uploadfilename = os.path.join(path, "." + name + ".upload")
-            with open(uploadfilename, 'w') as uploadfile:
+            with open(uploadfilename, 'w', encoding="ascii") as uploadfile:
                 # Put some contents into the file. Doesn't even matter what.
                 # The background scan won't touch anything with contents until
                 # it's at least 20 minutes old (when the writing process is
@@ -360,7 +362,7 @@ class StreamWriter:
             finished_filename = self.current_filename
             self.close_compressor()
             self.mark_finished(finished_filename)
-        except:
+        except Exception:
             self.logger.exception("Uncaught exception in write_loop")
         finally:
             self.write_loop_exited.set()
@@ -391,7 +393,8 @@ class StreamWriter:
                         # because of the hidden attribute. Windows doesn't like
                         # the 'w' mode on hidden files (need different flags
                         # passed to CreateFile).
-                        with open(uploadfilename, 'r+') as uploadfile:
+                        with open(uploadfilename, 'r+',
+                                  encoding="ascii") as uploadfile:
                             uploadfile.seek(0)
                             uploadfile.truncate()
 
@@ -407,7 +410,7 @@ class StreamWriter:
                     msg = "Uncollected compressor exited with code {}"
                     self.logger.warning(msg.format(ret_val))
                 self.compressors.clear()
-        except:
+        except Exception:
             self.logger.exception("Uncaught exception in monitor_loop")
 
     def close(self):
@@ -443,7 +446,13 @@ class StreamManager:
         self.got_packets = False
         self.finished = threading.Event()
         self.reset = threading.Event()
-        self.stream_thread = threading.Thread(target=self.stream_thread_run)
+
+        if len(streams_to_activate) == 0:
+            self.stream_thread = threading.Thread(
+                target=self.stream_thread_run_no_streams)
+        else:
+            self.stream_thread = threading.Thread(
+                target=self.stream_thread_run)
 
         self.writer = None
 
@@ -465,6 +474,43 @@ class StreamManager:
             self.writer.join()
             self.writer = None
 
+    def stream_thread_run_no_streams(self):
+        try:
+            if not self.disable_archiving:
+                self.writer = StreamWriter(
+                    self.logger, self.header_dict, self.device_name,
+                    self.base_dir, self.compression_level, self.upload_pipe)
+
+            self.status_pipe.send("connected")
+
+            self.device.set_connect_callback(self.connect_callback)
+
+            while True:  # loop while connected
+                if self.finished.wait(timeout=0.1):
+                    break
+
+                if hyperborea.proxy.device_lock.acquire(blocking=False):
+                    try:
+                        # just do a meaningless test command
+                        self.device.echo_params(b'')
+                    except asphodel.AsphodelError:
+                        self.status_pipe.send("error disconnect")
+                    finally:
+                        hyperborea.proxy.device_lock.release()
+            self.device.set_connect_callback(None)
+
+            # stop the writer
+            if self.writer:
+                self.writer.close()
+        except Exception:
+            self.logger.exception(
+                "Unhandled exception in stream_thread_run_no_streams")
+            self.status_pipe.send("error")
+
+            # stop the writer
+            if self.writer:
+                self.writer.close()
+
     def stream_thread_run(self):
         try:
             # flush any packets already on the device
@@ -482,6 +528,8 @@ class StreamManager:
 
             self.status_pipe.send("connected")
 
+            self.device.set_connect_callback(self.connect_callback)
+
             while not self.finished.is_set():  # loop while connected
                 self.device.poll_device(100)
                 if self.reset.is_set():
@@ -490,6 +538,8 @@ class StreamManager:
                         *self.stream_counts, callback=self.packet_callback)
                     self.reset.clear()
 
+            self.device.set_connect_callback(None)
+
             self.device.stop_streaming_packets()
 
             # do a final poll after stopping streaming to clean up
@@ -499,8 +549,11 @@ class StreamManager:
             if self.writer:
                 self.writer.close()
 
-            self.stop_streams()
-        except:
+            try:
+                self.stop_streams()
+            except asphodel.AsphodelError:
+                pass  # probably disconnected, ignore
+        except Exception:
             self.logger.exception("Unhandled exception in stream_thread_run")
             self.status_pipe.send("error")
 
@@ -560,3 +613,12 @@ class StreamManager:
                 self.packet_pipe.send(stream_packets)
             if self.writer:
                 self.writer.write(stream_packets)
+
+    def connect_callback(self, status, connected):
+        try:
+            if not connected:
+                # remove callback
+                self.device.set_connect_callback(None)
+                self.status_pipe.send("error disconnect")
+        except Exception:
+            self.logger.exception("Unhandled exception in connect_callback")
